classdef SessionData < handle
    %SESSIONDATA Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        filepath
        data
    end
    
    methods
        function obj = SessionData(filepath)
            obj.filepath = filepath;
            if (exist(filepath, 'file'))
                try
                    load(filepath);
                    if (exist('data','var'))
                        printc(mfilename,['loaded data from ' filepath]); 
                        obj.data = data;
                    else
                        printc(mfilename,['filename exists, no data: ' filepath]);
                        obj.data = []; 
                    end
                catch ME
                    printc(mfilename, ME.message);
                    obj.data = [];
                    
                end
                
            else
                obj.data = [];
                
            end
            
            obj.initSwitch();
        end
        
        function recordThisState(obj, session)
                obj.data.state = recordState(obj,session);
           
        end
        
        function state = recordState(obj, session)
           if(session.hasSystem())
                state = struct();
                state.system = session.getSystem().getName();
                state.diagram = session.getCurveManager().getDiagramName();
                
                c = session.getCurrentCurve();
                state.currentcurve = [];
                
                
                if (c.hasFile())
                   state.curvename = c.getLabel(); 
                else
                   state.curvename = []; 
                end
                
                state.curve = c.saveobj();
                state.startdata = session.getStartData().saveobj();
                state.contdata  = session.getContData().saveobj();
%              state.numdata = session.numericdata.saveobj();
                state.plotdata = session.plotmanager.saveobj();
                 
                state.branchselection = session.branchmanager.getSelectionIndex();
                
           else
                state = []; 
           end
           
        end       
        
        function restoreState(obj, session, optionalstate) 
            if (~obj.data.opt.restorestate.value) , return; end;
            
            if nargin == 3
               obj.data.state = optionalstate; 
            end
            
            if (isfield(obj.data, 'state') && ~isempty(obj.data.state))
               state = obj.data.state;
               try
               sys = SystemSpace.loadSystem(session , state.system);
               catch ME
                  printc(mfilename, ME.message);
                  obj.data.state = [];
                  return;
               end
               % sys loaded
               dlist = InspectorDiagram.getDiagramList( sys.getPath());
               if (ismember(state.diagram , dlist))
                   dpath = [sys.getPath() '/' state.diagram];
               else 
                  sys.setUp(); 
                  dpath = sys.getDefaultDiagramPath();
               end
               cm = CurveManager(session , dpath);
               
               if (isempty(state.curvename) || ~cm.curveExist(state.curvename))
                   curve = cm.getBlankCurve(obj);
                   curve.loadIn(state.curve);
                   
               else
                  curve =  cm.loadCurveFromMemory( state.curvename , state.curve );
               end
               
               
               session.changeCurve(sys  ,  cm, curve, state.startdata , state.contdata);

                  session.branchmanager.select(session ,  state.branchselection );

               
%             session.numericdata.copyOver(state.numdata);
               session.plotmanager.loadIn(session, state.plotdata);
               
            end
        end
        
        
        
        function saveData(obj)
            data = obj.data;
            save(obj.filepath,'data');            
        end

        
        function bool = getSwitch(obj , switchname)
              if isfield(obj.data.opt, switchname)
                    bool = obj.data.opt.(switchname).value;
              else
                    warning('getSwitched called with unknown switch');
                    bool = 0;
              end
        end
        function setSwitch(obj, switchname , bool)
            obj.data.opt.(switchname).value = bool;
        end
        
        function initSwitch(obj)
           if (~isfield(obj.data, 'opt'))
               obj.data.opt.restorestate = struct('tag', 'debug' , 'value' , 1);
               obj.data.opt.popup = struct('tag', 'debug' , 'value' , 1);
               obj.data.opt.debug =  struct('tag', 'debug' , 'value' , 0);
               obj.data.opt.autoredraw =  struct('tag', 'debug' , 'value' , 0);
               obj.data.opt.autofit = struct('tag', 'debug' , 'value' , 0);
               obj.data.opt.orbitdrawpointnumbers = struct('tag', 'draw' , 'value' , 1);
               obj.data.opt.orbitdrawlines = struct('tag', 'draw' , 'value' , 1);
	 
           end
                        
        end
        function tag = getSwitchTag(obj, switchname)
            tag = obj.data.opt.(switchname).tag;
        end
        function name =  getSwitchName(obj, switchname)
           names = struct('orbitdrawpointnumbers', 'Draw numbers when plotting orbits', ...
               'orbitdrawlines' , 'Draw lines when plotting orbits');
            
           if isfield(names, switchname)
              name = names.(switchname); 
           else
              name = switchname;
           end
            
        end

    end
    
end

